// Copyright 2025, IPng Networks GmbH, Pim van Pelt <pim@ipng.ch>

package main

import (
	"os"
	"path/filepath"
	"testing"
)

func TestConfigRead(t *testing.T) {
	tempDir := t.TempDir()

	// Create a single config file with types and devices
	configPath := filepath.Join(tempDir, "test-config.yaml")
	configContent := `types:
  test-type:
    commands:
      - show version
      - show status

devices:
  test-device:
    user: testuser
    type: test-type
  direct-device:
    user: directuser
    commands:
      - direct command
`

	err := os.WriteFile(configPath, []byte(configContent), 0644)
	if err != nil {
		t.Fatalf("Failed to create test config file: %v", err)
	}

	cfg, err := ConfigRead([]string{configPath})
	if err != nil {
		t.Fatalf("Failed to load config: %v", err)
	}

	// Test types section
	if len(cfg.Types) != 1 {
		t.Errorf("Expected 1 type, got %d", len(cfg.Types))
	}

	testType, exists := cfg.Types["test-type"]
	if !exists {
		t.Error("Expected 'test-type' to exist in types")
	}

	if len(testType.Commands) != 2 {
		t.Errorf("Expected 2 commands in test-type, got %d", len(testType.Commands))
	}

	// Test devices section
	if len(cfg.Devices) != 2 {
		t.Errorf("Expected 2 devices, got %d", len(cfg.Devices))
	}

	testDevice, exists := cfg.Devices["test-device"]
	if !exists {
		t.Error("Expected 'test-device' to exist in devices")
	}

	if testDevice.User != "testuser" {
		t.Errorf("Expected user 'testuser', got '%s'", testDevice.User)
	}

	if testDevice.Type != "test-type" {
		t.Errorf("Expected type 'test-type', got '%s'", testDevice.Type)
	}
}

func TestConfigReadMerging(t *testing.T) {
	tempDir := t.TempDir()

	// Create first config file with device types
	typesPath := filepath.Join(tempDir, "types.yaml")
	typesContent := `types:
  test-type:
    commands:
      - show version
      - show status`

	err := os.WriteFile(typesPath, []byte(typesContent), 0644)
	if err != nil {
		t.Fatalf("Failed to create types file: %v", err)
	}

	// Create second config file with devices
	devicesPath := filepath.Join(tempDir, "devices.yaml")
	devicesContent := `devices:
  test-device:
    user: testuser
    type: test-type`

	err = os.WriteFile(devicesPath, []byte(devicesContent), 0644)
	if err != nil {
		t.Fatalf("Failed to create devices file: %v", err)
	}

	// Load and merge configs
	cfg, err := ConfigRead([]string{typesPath, devicesPath})
	if err != nil {
		t.Fatalf("Failed to merge configs: %v", err)
	}

	// Check that merging worked
	if len(cfg.Types) != 1 {
		t.Errorf("Expected 1 type, got %d", len(cfg.Types))
	}

	testType, exists := cfg.Types["test-type"]
	if !exists {
		t.Error("Expected 'test-type' to exist in merged config")
	}

	if len(testType.Commands) != 2 {
		t.Errorf("Expected 2 commands in test-type, got %d", len(testType.Commands))
	}

	if len(cfg.Devices) != 1 {
		t.Errorf("Expected 1 device, got %d", len(cfg.Devices))
	}

	testDevice, exists := cfg.Devices["test-device"]
	if !exists {
		t.Error("Expected 'test-device' to exist in merged config")
	}

	if testDevice.Type != "test-type" {
		t.Errorf("Expected device type 'test-type', got '%s'", testDevice.Type)
	}
}

func TestConfigReadOverrides(t *testing.T) {
	tempDir := t.TempDir()

	// Create base config
	basePath := filepath.Join(tempDir, "base.yaml")
	baseContent := `devices:
  test-device:
    user: baseuser
    type: base-type`

	err := os.WriteFile(basePath, []byte(baseContent), 0644)
	if err != nil {
		t.Fatalf("Failed to create base file: %v", err)
	}

	// Create override config
	overridePath := filepath.Join(tempDir, "override.yaml")
	overrideContent := `devices:
  test-device:
    user: overrideuser`

	err = os.WriteFile(overridePath, []byte(overrideContent), 0644)
	if err != nil {
		t.Fatalf("Failed to create override file: %v", err)
	}

	// Load with override (later file should override earlier file)
	cfg, err := ConfigRead([]string{basePath, overridePath})
	if err != nil {
		t.Fatalf("Failed to merge configs: %v", err)
	}

	testDevice := cfg.Devices["test-device"]
	if testDevice.User != "overrideuser" {
		t.Errorf("Expected overridden user 'overrideuser', got '%s'", testDevice.User)
	}

	// Type should be preserved from base config
	if testDevice.Type != "base-type" {
		t.Errorf("Expected type 'base-type' to be preserved, got '%s'", testDevice.Type)
	}
}

func TestConfigReadInvalidFile(t *testing.T) {
	_, err := ConfigRead([]string{"/nonexistent/config.yaml"})
	if err == nil {
		t.Error("Expected error when loading nonexistent config file")
	}
}

func TestConfigReadInvalidYAML(t *testing.T) {
	tempDir := t.TempDir()
	configPath := filepath.Join(tempDir, "invalid-config.yaml")

	// Create a file with invalid YAML syntax
	invalidContent := `types:
  test-type:
    commands
      - invalid yaml`

	err := os.WriteFile(configPath, []byte(invalidContent), 0644)
	if err != nil {
		t.Fatalf("Failed to create invalid config file: %v", err)
	}

	_, err = ConfigRead([]string{configPath})
	if err == nil {
		t.Error("Expected error when loading invalid YAML")
	}
}

func TestConfigReadEmptyFile(t *testing.T) {
	tempDir := t.TempDir()
	configPath := filepath.Join(tempDir, "empty-config.yaml")

	// Create empty file
	err := os.WriteFile(configPath, []byte(""), 0644)
	if err != nil {
		t.Fatalf("Failed to create empty config file: %v", err)
	}

	cfg, err := ConfigRead([]string{configPath})
	if err != nil {
		t.Fatalf("Failed to load empty config: %v", err)
	}

	// Should have empty maps
	if len(cfg.Types) != 0 {
		t.Errorf("Expected 0 types in empty config, got %d", len(cfg.Types))
	}

	if len(cfg.Devices) != 0 {
		t.Errorf("Expected 0 devices in empty config, got %d", len(cfg.Devices))
	}
}

func TestConfigReadComplexMerge(t *testing.T) {
	tempDir := t.TempDir()

	// Create device types file
	typesPath := filepath.Join(tempDir, "types.yaml")
	typesContent := `types:
  srlinux:
    commands:
      - show version
      - show platform linecard
  eos:
    commands:
      - show version
      - show inventory`

	err := os.WriteFile(typesPath, []byte(typesContent), 0644)
	if err != nil {
		t.Fatalf("Failed to create types file: %v", err)
	}

	// Create production devices file
	prodPath := filepath.Join(tempDir, "production.yaml")
	prodContent := `devices:
  prod-asw100:
    user: netops
    type: srlinux
  prod-core-01:
    user: netops
    type: eos`

	err = os.WriteFile(prodPath, []byte(prodContent), 0644)
	if err != nil {
		t.Fatalf("Failed to create production file: %v", err)
	}

	// Create lab devices file
	labPath := filepath.Join(tempDir, "lab.yaml")
	labContent := `devices:
  lab-switch:
    user: admin
    type: srlinux
    commands:
      - show version only`

	err = os.WriteFile(labPath, []byte(labContent), 0644)
	if err != nil {
		t.Fatalf("Failed to create lab file: %v", err)
	}

	// Load merged configuration
	cfg, err := ConfigRead([]string{typesPath, prodPath, labPath})
	if err != nil {
		t.Fatalf("Failed to load merged config: %v", err)
	}

	// Verify types were loaded correctly
	if len(cfg.Types) != 2 {
		t.Errorf("Expected 2 types, got %d", len(cfg.Types))
	}

	srlinuxType, exists := cfg.Types["srlinux"]
	if !exists {
		t.Error("Expected 'srlinux' type to exist")
	}
	if len(srlinuxType.Commands) != 2 {
		t.Errorf("Expected 2 commands for srlinux type, got %d", len(srlinuxType.Commands))
	}

	// Verify devices reference the correct types
	if len(cfg.Devices) != 3 {
		t.Errorf("Expected 3 devices, got %d", len(cfg.Devices))
	}

	prodDevice, exists := cfg.Devices["prod-asw100"]
	if !exists {
		t.Error("Expected 'prod-asw100' device to exist")
	}
	if prodDevice.Type != "srlinux" {
		t.Errorf("Expected prod-asw100 type 'srlinux', got '%s'", prodDevice.Type)
	}

	labDevice, exists := cfg.Devices["lab-switch"]
	if !exists {
		t.Error("Expected 'lab-switch' device to exist")
	}
	if len(labDevice.Commands) != 1 {
		t.Errorf("Expected 1 custom command for lab-switch, got %d", len(labDevice.Commands))
	}
}

func TestConfigReadAddress(t *testing.T) {
	tempDir := t.TempDir()

	// Create config file with address field
	configPath := filepath.Join(tempDir, "address-config.yaml")
	configContent := `devices:
  router-with-address:
    user: testuser
    address: 192.168.1.100
  router-without-address:
    user: testuser`

	err := os.WriteFile(configPath, []byte(configContent), 0644)
	if err != nil {
		t.Fatalf("Failed to create test config file: %v", err)
	}

	cfg, err := ConfigRead([]string{configPath})
	if err != nil {
		t.Fatalf("Failed to load config: %v", err)
	}

	// Test device with address
	deviceWithAddress, exists := cfg.Devices["router-with-address"]
	if !exists {
		t.Error("Expected 'router-with-address' to exist")
	}
	if deviceWithAddress.Address != "192.168.1.100" {
		t.Errorf("Expected address '192.168.1.100', got '%s'", deviceWithAddress.Address)
	}

	// Test device without address (should be empty)
	deviceWithoutAddress, exists := cfg.Devices["router-without-address"]
	if !exists {
		t.Error("Expected 'router-without-address' to exist")
	}
	if deviceWithoutAddress.Address != "" {
		t.Errorf("Expected empty address, got '%s'", deviceWithoutAddress.Address)
	}
}
